package ambientMorality;



import java.util.Random;
import java.util.Vector;


import rayTrace.Box;
import rayTrace.Collection;
import rayTrace.Element;
import rayTrace.Ray;
import support.AsciiMatrixFile;
import support.BinaryMatrixFile;
import support.BinaryMatrixWriter;



public class AmbientMorality {
	final double reHitTolerence = 1e-6;
	final String rootDir = "/work/lidr/rays";
	
	public AmbientMorality(String args[]) {
		//test2();
		//if(true)return;
		
		boolean doTraces = true;
		
		int nPoints = 30;
		int nHits = 20;
		int nTheta = 300;
		int nPhi = 300;
		
		double theta0 = -4 * Math.PI / 180.0;
		double theta1 = 4 * Math.PI / 180.0;
		double phi0 = -8 * Math.PI / 180.0;
		double phi1 = 2 * Math.PI / 180.0; //*/
		
		/*double theta0 = -10 * Math.PI / 180.0;
		double theta1 = 10 * Math.PI / 180.0;
		double phi0 = -10 * Math.PI / 180.0;
		double phi1 = 10 * Math.PI / 180.0;*/
		
		Box boundBox = new Box(new double[]{-15, -5, -5}, new double[]{ 50, 5, 30}, false,1,1, reHitTolerence);
	
		Collection wins1 = Collection.loadDiscs(rootDir + "/windowDiscsInner.txt", reHitTolerence);	//inner window tube ends
		Collection wins2 = Collection.loadDiscs(rootDir + "/windowDiscsOuter.txt", reHitTolerence);	//out window tube ends
		Collection verts = Collection.loadDishes(rootDir + "/vertMirrors.txt", reHitTolerence);	//verticle mirror assembly
		Collection horiz = Collection.loadDishes(rootDir + "/horizMirrors.txt", reHitTolerence);	//horizonal mirror assembly
		Collection lasMirror = Collection.loadDiscs(rootDir + "/laserMirror.txt", reHitTolerence);
		Collection lasBlock = Collection.loadDiscs(rootDir + "/laserBlock.txt", reHitTolerence);
		Collection roofHoles = Collection.loadDiscs(rootDir + "/roofHoles.txt", reHitTolerence);
		Element roof1 = roofHoles.elems[0];
		Element roof2 = roofHoles.elems[1];
		
		
		Collection all = new Collection(new Element[]{ verts, horiz, wins1, wins1, lasMirror, lasBlock, roofHoles} );
		
				
		double dTheta = (theta1 - theta0)/(nTheta - 1.0);
		double dPhi = (phi1 - phi0)/(nPhi - 1.0);
		
		double minTheta=Double.POSITIVE_INFINITY;
		double maxTheta=Double.NEGATIVE_INFINITY;
		double minPhi=Double.POSITIVE_INFINITY;
		double maxPhi=Double.NEGATIVE_INFINITY;
		
		
		for(int setIdx=0;setIdx<21;setIdx++){
		//int setIdx = 0; {
			
			//double shift = setIdx * 0.005 - 0.03;
			//verts = Collection.loadDishes(rootDir + "/vertMirrors.txt", reHitTolerence);	//verticle mirror assembly
			//verts.shift(new double[]{ 0, 0, shift });
			
			double R1[] = new double[]{7.8295, 0.0, 0.0}; //centre of input window (* 200'C conversion)
			//double R2[] = new double[]{1.7342, 0.010, 0.2966 }; //beam dump from marc's new window drawing diagram (* 200'C conversion)
			double shift = setIdx * 0.10 / 20.0 - 0.05;
			double R2[] = new double[]{1.7342, 0.010, 0.2966 + shift}; //beam dump from marc's new window drawing diagram (* 200'C conversion)
			
			/*double shift = setIdx * 0.5 - 5.0;
			lasMirror = Collection.loadDiscs(rootDir + "/laserMirror.txt", reHitTolerence);
			double x[] = new double[3];
			System.arraycopy(lasMirror.elems[0].centre, 0, x, 0, 3);
			lasMirror.rotateOnY(x, shift * Math.PI / 180.0); //*/
			
			System.out.println("Starting set "+setIdx+" at "+shift);
			

			double U[] = new double[3];
			double losStart[]  = new double[3];
			double losEnd[]  = new double[3];
			double losLen[]  = new double[3];
			double losStep[]  = new double[3];
			
			for(int i=0;i<3;i++){
				R1[i] = R1[i] * 1.0022; 
				R2[i] = R2[i] * 1.0022;
				U[i] = R2[i] - R1[i];
				losStart[i] = R1[i] + U[i] * ( (4.5 - R1[0]) / U[0]);
				losEnd[i] = R2[i] + U[i] * ( (1.5 - R2[0]) / U[0]);
				losLen[i] = losEnd[i] - losStart[i];
				losStep[i] = losLen[i] / (nPoints-1);
			}
			
			double losData[][] = new double[nPoints][4];
	
			for(int losIdx=0;losIdx<nPoints;losIdx++)
			//int losIdx=20;
			{
				
				for(int i=0;i<3;i++)
					losData[losIdx][i] = losStart[i] + losIdx * losStep[i];
				
				System.out.print("(" + losData[losIdx][0] + ", " + losData[losIdx][1] + ", " + losData[losIdx][2] + "):");
				
				double hitData[][] = new double[nTheta * nPhi][3];
				int j=0;
				double thisRay[][] = new double[10][3];
				
				BinaryMatrixWriter allRays=null,completeRays=null;
				if(doTraces){
					allRays = new BinaryMatrixWriter("/work/scrap/amAllRays-"+losIdx+".bin",3);
					completeRays = new BinaryMatrixWriter("/work/scrap/amCompleteRays-"+losIdx+".bin",3);
				}
				
				
				for(int thetaIdx=0;thetaIdx<nTheta;thetaIdx++){
					double theta = theta0 + thetaIdx * dTheta;
					for(int phiIdx=0;phiIdx<nPhi;phiIdx++){
						double phi = phi0 + phiIdx * dPhi;
						
						double start[] = new double[3];
						System.arraycopy(losData[losIdx],0,start,0,3);
						
						double unit[] = new double[]{ Math.cos(theta)*Math.cos(phi),
														Math.sin(theta),
														Math.cos(theta)*Math.sin(phi) };
						
						Ray ray = new Ray(start,unit);
						 System.arraycopy(ray.start,0,thisRay[0],0,3);
						
						do{ //just for the break
							if(ray.traceRay(wins1,1,null)==null) break; if(doTraces)System.arraycopy(ray.start,0,thisRay[1],0,3);
							if(ray.traceRay(wins2,1,null)==null) break; if(doTraces)System.arraycopy(ray.start,0,thisRay[2],0,3);
							
							if(ray.traceRay(lasBlock,1,null)!=null) break; if(doTraces)System.arraycopy(ray.start,0,thisRay[3],0,3);
							
							if(ray.traceRay(verts,1,null)==null) break; if(doTraces)System.arraycopy(ray.start,0,thisRay[4],0,3);
							if(ray.traceRay(lasMirror,1,null)==null) break; if(doTraces)System.arraycopy(ray.start,0,thisRay[5],0,3);
							if(ray.traceRay(horiz,1,null)==null) break; if(doTraces)System.arraycopy(ray.start,0,thisRay[6],0,3);
							
							if(ray.traceRay(roof1,1,null)==null) break; if(doTraces)System.arraycopy(ray.start,0,thisRay[7],0,3);
							if(ray.traceRay(roof2,1,null)==null) break; if(doTraces)System.arraycopy(ray.start,0,thisRay[8],0,3);
						
						
							//we've hit the final plane, count it
							losData[losIdx][3]++; 
							
							//store the final plane contact position
							System.arraycopy(ray.start,0,hitData[j++],0,3);
												
							//move on by 5m to show past the final plane
							if(doTraces){
								for(int i=0;i<3;i++)
									ray.start[i] += ray.unit[i] * 5;
							
								System.arraycopy(ray.start,0,thisRay[9],0,3);
							
								//store all the points of this complete ray
								for(int i=0;i<thisRay.length;i++)
									completeRays.writeRow(thisRay[i]);
							}
							
							if(theta<minTheta)minTheta = theta;
							if(theta>maxTheta)maxTheta = theta;
							if(phi<minPhi)minPhi = phi;
							if(phi>maxPhi)maxPhi = phi;
							
						}while(false);
						
						//store all the points of this possibly incomplete ray
						if(doTraces)
							for(int i=0;i<thisRay.length;i++)
								allRays.writeRow(thisRay[i]);
						
											
	
						
					}
					System.out.print(".");
				}		
				System.out.println("");
				System.out.println(losData[losIdx][3]);
	
				if(doTraces){
					allRays.close();
					completeRays.close();
				}
				
				BinaryMatrixFile.mustWrite("/work/scrap/amOut-"+losIdx+".bin",hitData,false);
			}
			
			System.out.println("minTheta = " + minTheta*180/Math.PI + ", maxRange=" + theta0);
			System.out.println("maxTheta = " + maxTheta*180/Math.PI + ", maxRange=" + theta1);
			System.out.println("minPhi = " + minPhi*180/Math.PI + ", maxRange=" + phi0);
			System.out.println("maxPhi = " + maxPhi*180/Math.PI + ", maxRange=" + phi1);
			
			
			AsciiMatrixFile.mustWrite("/work/lidr/rays/amLosCount-los"+setIdx+".txt", losData,false);
		}
	}
	
	
	
	public static void main(String[] args) { new AmbientMorality(args); }

	public void test3(Collection coll){

		double start[] = new double[]{ 3.052, 0.0079, 0.2333};
		double unit[] = new double[]{ 7.4415, 0, -0.1884};
		
		double sum=0;
		for(int i=0;i<3;i++){
			unit[i] -= start[i];
			sum += unit[i] * unit[i];
		}
		sum = Math.sqrt(sum);
		
		for(int i=0;i<3;i++)
			unit[i] /= sum;
		
		Ray ray = new Ray(start,unit);
		
		double hits[][] = ray.traceRay(coll,50,null);
		
		for(int i=0;i<50;i++)
			if(hits[i]!=null)
				System.out.println(i + ":\t" + hits[i][0] + "\t" + hits[i][1] + "\t" + hits[i][2]);
				
		
	}
	
	public void test2(){
		int nHits = 40;
		int nRays = 10000;
		
		
		Box boundBox = new Box(new double[]{ -5, -10, -5}, new double[]{ 20, 10, 30}, false,1,1, reHitTolerence);
		Element elems[] = OpticElementsFile.read("/work/lidr/rays/sorted/optics3.txt");
		
		Collection coll = new Collection(elems);
		//coll.addElement(boundBox);
				
		//dish1.rotateOnZ(new double[]{-1,0,0}, 90 * Math.PI / 180);
		
		//double hitData[][] = new double[nRays * nHits][3];
		Random randGen = new Random();
		BinaryMatrixWriter hitWriter = new BinaryMatrixWriter("/work/scrap/amKe3Out.bin",3);
		
		test3(coll); if(true)return;
		
		
		double minTheta=Double.POSITIVE_INFINITY;
		double maxTheta=Double.NEGATIVE_INFINITY;
		double minPhi=Double.POSITIVE_INFINITY;
		double maxPhi=Double.NEGATIVE_INFINITY;
		
		
		
		int hitDataIdx=0;
		for(int i=0;i<nRays;i++){
			double theta = randGen.nextDouble() * 8 - 4;
			double phi = randGen.nextDouble() * 8 - 7;
			
			theta *= Math.PI / 180.0;
			phi *= Math.PI / 180.0;
			//theta=0;

			/*double start[] = new double[]{ 8.96, 0, 19.4};
			double unit[] = new double[]{ Math.cos(theta)*Math.sin(phi),
											Math.sin(theta),
											Math.cos(theta)*Math.cos(phi), }; //*/
			
			double start[] = new double[]{ 3.0517241, 0.0078667, 0.2333269};
			//double start[] = new double[]{ 2.1206897, 0.0093942, 0.2786314};
			double unit[] = new double[]{ Math.cos(theta)*Math.cos(phi),
											Math.sin(theta),
											Math.cos(theta)*Math.sin(phi) }; //*/
			
			Ray ray = new Ray(start, unit );
			
			double hits[][] = ray.traceRay(coll,nHits,elems[17]);
			if(hits != null){
				
				for(int j=0;j<nHits;j++)
					if(hits[j] != null)
						hitWriter.writeRow(hits[j]);
						//System.arraycopy(hits[j],0,hitData[hitDataIdx++],0,3);

				//hitData[hitDataIdx] = new double[]{ 0,0,0 };
				hitDataIdx++;
				hitWriter.writeRow(new double[]{ 0,0,0 });
				//System.out.print(".");

				if(theta<minTheta)minTheta = theta;
				if(theta>maxTheta)maxTheta = theta;
				if(phi<minPhi)minPhi = phi;
				if(phi>maxPhi)maxPhi = phi;
			}
			
			if( ((i * 1000.0 / nRays) % 10) == 0 )
				System.out.println((i * 100.0 / nRays)+": " + hitDataIdx);
			
		}
		//AsciiMatrixFile.mustWrite("/work/scrap/amTest.txt",hitData);
		hitWriter.close();

		System.out.println(hitDataIdx);
		
		System.out.println("minTheta = " + minTheta*180/Math.PI);
		System.out.println("maxTheta = " + maxTheta*180/Math.PI);
		System.out.println("minPhi = " + minPhi*180/Math.PI);
		System.out.println("maxPhi = " + maxPhi*180/Math.PI);
	}
	
	public void test4(Collection coll){

		double start[] = new double[]{ 3.052, 0.0079, 0.2333};
		double unit[] = new double[]{ 7.4415, 0, -0.1884};
		
		double sum=0;
		for(int i=0;i<3;i++){
			unit[i] -= start[i];
			sum += unit[i] * unit[i];
		}
		sum = Math.sqrt(sum);
		
		for(int i=0;i<3;i++)
			unit[i] /= sum;
		
		Ray ray = new Ray(start,unit);
		
		boolean merp = attemptRay(ray,null,null);
		
		/*double hits[][] = ray.traceRay(coll,60,null);
		
		for(int i=0;i<50;i++)
			if(hits[i]!=null)
				System.out.println(i + ":\t" + hits[i][0] + "\t" + hits[i][1] + "\t" + hits[i][2] + "\t" + hits[i][3]);
			//*/	
		
	}
	
	
}
