package ambientMorality;

import java.io.*;
import java.util.Vector;

import rayTrace.*;

public class OpticElementsFile {
	
	static class ElemInfo{
		final static int TYPE_UNKNOWN = 0;
		final static int TYPE_DISC = 1;
		final static int TYPE_SQUARE = 2;
		final static int TYPE_DISH = 3;
		final static int TYPE_IRIS = 4;
		
		double centre[] = null;
		double normal[] = null;
		double vector[] = null;
		double length = Double.NaN;
		double width = Double.NaN;
		double diameter = Double.NaN;
		double radOfCurv = Double.NaN;
		double aperture = Double.NaN;
		String matFront, matBack;
		int type = TYPE_UNKNOWN;
		int matType = Element.MATTYPE_UNKNOWN;
		int surfaceIndex = -1;
		String CollectionID = null;
	};
	
	public static Element[] read(String fileName) {

		Vector <Element>elements = new Vector<Element>();
		
		ElemInfo elInf = new ElemInfo();
		String collectionID = null;
		
		try{
			FileInputStream fin = new FileInputStream(fileName);
			BufferedReader bin = new BufferedReader(new InputStreamReader(fin));
		
			do{
				String line = null;
				String procLine = null;
			
				do{
					line = bin.readLine();
					if(line==null)break;
					
					procLine = line.replaceAll("\\s+", "\t"); //multiple spaces -> tab
					procLine = procLine.replaceAll(":", "\t");	//colon to tab
					procLine = procLine.replaceAll("^\\t+", ""); //tabs at beggining to nothing
					procLine = procLine.replaceAll("\\t+$", ""); //tabs at end to nothing
					procLine = procLine.replaceAll("\\t+", "\t");  //mulitple tabs to 1
				}while(line.length()==0 || line.startsWith("//"));
				
				if(line==null)break;
				
				
				String fields[] = procLine.split("\t");
				int n= fields.length;
				double dp[] = new double[n];
				
				for(int i=0;i<n;i++)
					try{
						dp[i] = Double.parseDouble(fields[i]);
					}catch(NumberFormatException err){ dp[i] = Double.NaN; }
					
				if(!line.startsWith("\t") && !line.startsWith(" ")){
					collectionID = line;
					continue;
				}
				
				if(fields[0].equalsIgnoreCase("Surface") && fields[1].equalsIgnoreCase("No")){
					if(elInf.surfaceIndex >=0 ){
						Element newElem = processElement(elInf);
						if(newElem != null)
							elements.add(newElem);
					}
						
					elInf = new ElemInfo();					
					elInf.CollectionID  = collectionID;
					//and start the new surface
					elInf.surfaceIndex = (int)dp[2];
					
					elInf.type = ElemInfo.TYPE_UNKNOWN;
					if(fields[3].equalsIgnoreCase("Flat") ){
						if( fields[4].equalsIgnoreCase("Circular") )
							elInf.type = ElemInfo.TYPE_DISC;
						else if( fields[4].equalsIgnoreCase("Rectangular") )
							elInf.type = ElemInfo.TYPE_SQUARE;						
					}else if(fields[3].equalsIgnoreCase("Spherical") ){
						if( fields[4].equalsIgnoreCase("Circular") )
							elInf.type = ElemInfo.TYPE_DISH;
					}else if(fields[3].equalsIgnoreCase("Iris") && fields[4].equalsIgnoreCase("Circular"))
						elInf.type = ElemInfo.TYPE_IRIS;
						
					
					elInf.matType = Element.MATTYPE_UNKNOWN;
					if( fields[5].equalsIgnoreCase("Reflecting"))
						elInf.matType = Element.MATTYPE_REFLECTING;
					else if( fields[5].equalsIgnoreCase("Reflective"))
						elInf.matType = Element.MATTYPE_REFLECTING;
					else if( fields[5].equalsIgnoreCase("Refractive"))
						elInf.matType = Element.MATTYPE_TRANSPARENT;
					else if( fields[5].equalsIgnoreCase("Absorbing"))
						elInf.matType = Element.MATTYPE_ABSORBING;
					else if( fields[5].equalsIgnoreCase("Low") && fields[6].equalsIgnoreCase("Pass"))
						elInf.matType = Element.MATTYPE_TRANSPARENT;
						
					
				}else if(fields[0].equalsIgnoreCase("Centre") || fields[0].equalsIgnoreCase("Center")){
					elInf.centre = new double[]{ dp[1], dp[2], dp[3] };
				}else if(fields[0].equalsIgnoreCase("Normal")){
					elInf.normal = new double[]{ dp[1], dp[2], dp[3] };
				}else if(fields[0].equalsIgnoreCase("Vector")){
					elInf.vector = new double[]{ dp[1], dp[2], dp[3] };
				}else if(fields[0].equalsIgnoreCase("Length/width")){
					elInf.length = dp[1];
					elInf.width = dp[2];
				}else if(fields[0].equalsIgnoreCase("Diameter")){
					elInf.diameter = dp[1];
				}else if(fields[0].equalsIgnoreCase("R-curvature")){
					elInf.radOfCurv = dp[1];
				}else if(fields[0].equalsIgnoreCase("Aperture")){					
					elInf.aperture = dp[1];
				}else if(fields[0].equalsIgnoreCase("Materials")){
					elInf.matFront = new String(fields[2]);
					elInf.matBack = new String(fields[1]);
				}

			}while(true);
			
			//and deal with the last one (if there was any)
			if(elInf.surfaceIndex >=0 ){
				Element newElem = processElement(elInf);
				if(newElem != null)
					elements.add(newElem);
			}
			
			
			Element elemArr[] = new Element[elements.size()];
			
			for(int i=0;i<elements.size();i++)
				elemArr[i] = elements.elementAt(i);
			
			return elemArr;
		
		}catch(IOException err){ throw new RuntimeException(err); }

	}
	
	public static Element processElement(ElemInfo elemInf){
		Element elem = null;
		double reHitTolerence = 1e-6;
				
		double rIdxFront = glasTab(elemInf.matFront);
		double rIdxBack = glasTab(elemInf.matBack);
		
		Ray.reNorm(elemInf.normal); //why do people insist on using 4dp?
		
		switch(elemInf.type){
			case ElemInfo.TYPE_DISC:
				elem = new Disc(elemInf.centre,elemInf.normal,elemInf.diameter/2.0,elemInf.matType,rIdxFront,rIdxBack,reHitTolerence);
				break;
			case ElemInfo.TYPE_IRIS:
				elem = new Iris(elemInf.centre,elemInf.normal,elemInf.diameter/2.0,elemInf.aperture / 2.0,elemInf.matType,rIdxFront,rIdxBack,reHitTolerence);
				break;
			case ElemInfo.TYPE_SQUARE:
				elem = new Square(elemInf.centre,elemInf.normal,elemInf.vector,elemInf.length,elemInf.width,elemInf.matType,rIdxFront,rIdxBack,reHitTolerence);				
				break;
			case ElemInfo.TYPE_DISH:
				if(elemInf.radOfCurv < 0){
					for(int i=0;i<3;i++)
						elemInf.normal[i] *= -1;
					elemInf.radOfCurv *= -1;
					double tmp = rIdxBack;
					rIdxBack = rIdxFront;
					rIdxFront = tmp;
				}
				elem = new Dish(elemInf.centre,elemInf.normal,elemInf.radOfCurv,elemInf.diameter,elemInf.matType,rIdxFront,rIdxBack,reHitTolerence);
				break;
			default:
			System.out.println("Unknown object type in collection " + elemInf.CollectionID);
			
		}
		
		if(elem != null) //and ste the element's ID to the surface index
			elem.ID = elemInf.surfaceIndex;
				
		return elem;
		
	}
	
	public static double glasTab(String matName){
		if(matName == null)return 1.0;
		
		if(matName.equalsIgnoreCase("SPEC"))
			return 1.4555;
		else if(matName.equalsIgnoreCase("BK7"))
			return 1.5131;
		else if(matName.equalsIgnoreCase("SF2"))
			return 1.6394;
		else if(matName.equalsIgnoreCase("VACUUM"))
			return 1.0;
		else if(matName.equalsIgnoreCase("VAC"))
			return 1.0;
		else return 1.0;
		
	}
	
	public static void main(String args[]){ read("/work/lidr/rays/sorted/optics3.txt");	}
}
