package ambientMorality;

import java.io.*;
import java.text.DecimalFormat;
import java.util.Vector;

public class SVGWriter {
	
	private PrintStream svgStream = null;

	public SVGWriter(String fileName){
		
		FileOutputStream file;
		try {
			file = new FileOutputStream(fileName);
			svgStream = new PrintStream(file);
		
			svgStream.println("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n" +
			"<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 03December 1999//EN\" \"http://www.w3.org/Graphics/SVG/SVG-19991203.dtd\">\n" +
			"<svg xml:space=\"preserve\" width=\"100cm\" height=\"100cm\" viewBox=\"0 -2 50 25\">\n" +
			"<style type=\"text/css\"> .ray {fill:none;stroke:black;stroke-width:0.01;} </style>\n" +
			"\n\n");
			
		} catch (FileNotFoundException e) {
			System.err.println("Unable to open SVG file '"+fileName+"': " + e.getMessage());
			svgStream = null;
		}
	}
	
	public void destroy(){
		if(svgStream==null)return;
		
		svgStream.println("\n\n</svg>");
		svgStream.close();
		svgStream=null;
		
	}

	public void addLine(Vector <double[]>points){
		if(svgStream==null)return;
		
		DecimalFormat fmt = new DecimalFormat("##.####");
		
		double x[] = points.elementAt(0);
		svgStream.print("<path class=\"ray\" d=\"M "+fmt.format(x[0])+" "+fmt.format(x[2])+" ");
		for(int i=1;i<points.size();i++){
			x = points.elementAt(i);
			svgStream.print("L "+fmt.format(x[0])+" "+fmt.format(x[2])+" ");
		}
		svgStream.println("\" />");
		
		x = points.elementAt(0);
		svgStream.print("<path class=\"ray\" d=\"M "+fmt.format(x[0]+20)+" "+fmt.format(x[1])+" ");
		for(int i=1;i<points.size();i++){
			x = points.elementAt(i);
			svgStream.print("L "+fmt.format(x[0]+20)+" "+fmt.format(x[1])+" ");
		}
		svgStream.println("\" />");
	}
		
	
}
