package rayTrace;



public abstract class Element {
	
	public final static int MATTYPE_UNKNOWN = 0;
	public final static int MATTYPE_REFLECTING = 1;
	public final static int MATTYPE_TRANSPARENT = 2;
	public final static int MATTYPE_ABSORBING = 3;
	
	protected double centre[] = new double[3];
	int matType = MATTYPE_UNKNOWN ;
	protected double rIndexRatio = 1; //ratio of rafractive index on normal side over refractive index on opposite side
	
	public int ID = -1; //an arbitary ID field for keeping track 
	
	public abstract Intersection findIntersection(Ray ray);
	
	public abstract void shift(double dX[]);
	
	public abstract void rotate(double point[], double matrix[][]);
		
	public abstract double boundingSphereRadius();
	
	public void rotateOnX(double point[], double ang){
		double mat[][] = new double[][]{ 	
				{1, 0, 0},
				{0, Math.cos(ang), -Math.sin(ang)},
				{0, Math.sin(ang), Math.cos(ang)} };
				
		rotate(point,mat);
	}
	public void rotateOnY(double point[], double ang){
		double mat[][] = new double[][]{ 	
				{Math.cos(ang), 0, -Math.sin(ang)},
				{0, 1, 0},
				{Math.sin(ang), 0, Math.cos(ang)} };
				
		rotate(point,mat);
	}
	public void rotateOnZ(double point[], double ang){
		double mat[][] = new double[][]{ 	
				{Math.cos(ang), -Math.sin(ang), 0},
				{ Math.sin(ang), Math.cos(ang), 0},
				{0, 0, 1} };				
		rotate(point,mat);
	}	
		
	public final boolean isTransparent(){
		return matType == MATTYPE_TRANSPARENT;
	}
	
	public final boolean isAbsorbing(){
		return matType == MATTYPE_ABSORBING;
	}
	
	public final boolean isReflective(){
		return matType ==MATTYPE_REFLECTING;
	}
	
	public static double[] normaliseVector(double vec[]){
		double sum=0;
		
		for(int i=0;i<vec.length;i++)
			sum += vec[i] * vec[i];
		
		sum = Math.sqrt(sum);
		
		for(int i=0;i<vec.length;i++)
			vec[i] /= sum;
		
		return vec;
	}
	
	public static Element findElement(Element elems[], int ID){
		for(int i=0;i<elems.length;i++)
			if(elems[i].ID == ID)
				return elems[i];
		
		return null;
	}

	

	public double[] getCentre() {
		return new double[]{ centre[0], centre[1], centre[2] } ;
	}
	public void setCentre(double[] centre) {
		System.arraycopy(centre,0,this.centre,0,3);		
	}
	public int getID() {
		return ID;
	}
	public void setID(int id) {
		ID = id;
	}
	public int getMatType() {
		return matType;
	}
	public void setMatType(int matType) {
		this.matType = matType;
	}
	public double getRIndexRatio() {
		return rIndexRatio;
	}
	public void setRIndexRatio(double indexRatio) {
		rIndexRatio = indexRatio;
	}
	
}
