package rayTrace;



public class Square extends Plane {
	
	private double lengthDir[] = new double[3];
	private double length;
	private double width;
	private double NxV[];
	
		
	public Square(double centre[], double normal[], double lengthDir[], double length, double width, int matType, 
			double frontRIndex, double backRIndex, double reHitTolerence) {
		super(centre,normal,matType,frontRIndex,backRIndex,reHitTolerence);
		this.lengthDir = lengthDir;
		this.length = length;
		this.width = width;
		
		NxV = Ray.cross(normal,lengthDir);		
	}
	
	public double boundingSphereRadius(){
		return Math.sqrt(width*width + length*length)/2;		
	} 
	
	
	public void rotate(double point[], double matrix[][]){
		super.rotate(point, matrix); //plane's rotator will rotate centre[] and normal[]
		
			
		double newLengthDir[] = new double[3];
		
		for(int i=0;i<3;i++)
			for(int j=0;j<3;j++)
				newLengthDir[i] += matrix[i][j] * lengthDir[j];
				
		lengthDir = newLengthDir;
	}
	@Override
	public Intersection findIntersection(Ray ray) {
			
		Intersection hit = super.findIntersection(ray); //find the place where the ray hits us (as if we were an infinite plane)
		
		if(hit == null || hit.distAlongRay <= 0) //didn't hit the plane (probably parallel)
			return null;		
	
		double paraInSquare = 0, perpInSquare = 0;
		
		for(int i=0;i<3;i++){
			paraInSquare += (hit.pos[i] - centre[i]) * lengthDir[i];
			perpInSquare += (hit.pos[i] - centre[i]) * NxV[i];			
		}
		
		hit.object = this;
		
		if( Math.abs(paraInSquare) <= (length/2.0) && Math.abs(perpInSquare) <= (width/2.0))
			return hit;
			
		
		return null;
	}
	
	
	/*
	 * //test for intersection of a ray and a square
		function [xPos,xNorm,dist] = squareIntersect(ray, sqr)
			[xPos,dist]=planeIntersect(ray,sqr.centre,sqr.normal);
		
			if dist < 0  //contact is backwards in time
				xNorm = [0,0,0];
				return
			end
			//N x V is the other vector in the plane
			NxV = zeros(1,3);
			NxV(1) =  sqr.normal(2) * sqr.vector(3) - sqr.normal(3) * sqr.vector(2);
			NxV(2) = -sqr.normal(1) * sqr.vector(3) + sqr.normal(3) * sqr.vector(1);
			NxV(3) =  sqr.normal(1) * sqr.vector(2) - sqr.normal(2) * sqr.vector(1);
		
			//find the position in the sqr's coords (para & perp to v)
			paraInSqaure = sum( (xPos - sqr.centre) .* sqr.vector );
			perpInsqr = sum( (xPos - sqr.centre) .* NxV );
		
			//are we inside width & height of sqr
			if abs(paraInSqaure) <= (sqr.len/2) & abs(perpInsqr) <= (sqr.width/2) 		
				xNorm = sqr.normal; //intersection normal is just plane normal		
				
			else
				xNorm = [0,0,0]; //otherwise it didnt hit
				
			end
		endfunction
	 */

}
